import gc
from time import time

from nozdormu.util import format_time

class BenchBatch:
    """This class contains benchmark to run"""

    def __init__(self, methodName):
        self.totalTime = 0.0
        self.count = 0
        self.methodName = methodName
        try:
            self.method = getattr(self, self.methodName)
        except:
            raise ImportError('Failed to load {}'.format(self.methodName))

    def __repr__(self):
        return self.methodName

    def run(self):
        """Peform setup/run method/teardown and time it"""
        # Manual GC
        gc.collect()

        # Warm up
        self.setUp()
        self.method()
        self.tearDown()

        # Timed run
        self.setUp()
        start = time()
        self.method()
        self.totalTime += time() - start
        self.tearDown()

        self.count += 1

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def exact(self):
        """Return the raw time per execution"""
        return self.totalTime / self.count

    def results(self):
        """Return human-readable output"""
        return '{} ({} / {} runs)'.format(format_time(self.exact()),
                format_time(self.totalTime), self.count)




class BenchSuite:
    """A suite that houses benchmarks"""

    def __init__(self, benchs=()):
        self.benchs = benchs
        self.batch = None

    def __iter__(self):
        return iter(self.benchs)

    def __repr__(self):
        return str(self.batch.__name__)




def format_time(arg):
    """Format timespans from seconds to smaller units with the right units"""
    if arg <= 1e-6:
        return '{}ns'.format(round(arg * 1000000000))
    elif arg <= 1e-3:
        return '{}μs'.format(round(arg * 1000000))
    elif arg <= 1:
        return '{}ms'.format(round(arg * 1000))
    else:
        return '{}s'.format(round(arg))




from setuptools import setup

import nozdormu

def readme():
    with open('readme.rst') as f:
        return f.read()

setup(name='nozdormu',
      version=nozdormu.VERSION,
      description='Python benchmarking for humans and dragons',
      long_description=readme(),
      url='https://github.com/sulami/nozdormu',
      author='Robin Schroer',
      author_email='sulami@peerwire.org',
      license='MIT',
      packages=['nozdormu'],
      zip_safe=False,
      )

